Barn = {};

local L = {};
local GARRISON_MAP_ID, NPC_NAME;
local icon = "Interface\\GossipFrame\\VendorGossipIcon";
local changeIcon = {
    "ORDER_FUR",
    "ORDER_FUR_LARGE",
    "ORDER_LEATHER",
    "ORDER_LEATHER_LARGE",
    "ORDER_MEAT",
    "ORDER_MEAT_LARGE"
};

Barn.init = function()
    L = Barn.L;
    
    local faction = strupper(UnitFactionGroup("player"));
    if(faction == "ALLIANCE") then
        GARRISON_MAP_ID = 971;
        NPC_NAME = L["NPC_NAME_ALLIANCE"];
    elseif(faction == "HORDE") then
        GARRISON_MAP_ID = 976;
        NPC_NAME = L["NPC_NAME_HORDE"];
    else
        Barn.error.factionError();
        return;
    end
   
    local isLocaleSupported = pcall(function()
        local test = L["ORDER_FUR"];
    end);
        
    if(not isLocaleSupported) then
        Barn.errors.localeError();
        return;
    end
    
    Barn.frame = CreateFrame("Frame", "BarnFrame", UIParent);
    Barn.frame:SetScript("OnEvent", Barn.OnEvent);
    Barn.frame:RegisterEvent("GOSSIP_SHOW");
end

Barn.OnEvent = function()
    local success, err = pcall(Barn.GossipShow);
    if(not success) then
        Barn.errors.gossipShowError(err);
    end
end

Barn.GossipShow = function()
    local isUsingGarrisonBarn = (GetCurrentMapAreaID() == GARRISON_MAP_ID) and (Barn.getNPCName() == NPC_NAME);
    
    if(isUsingGarrisonBarn) then
        for i = 1, GetNumGossipOptions() do
            local btn = _G["GossipTitleButton" .. i];
            local btnIcon = _G[btn:GetName() .. "GossipIcon"]
            Barn.applyOverride(btn, btnIcon);
        end
    end
end

Barn.applyOverride = function(btn, btnIcon)
    local txt = btn:GetText();
    local key = L[txt];

    if(txt ~= nil and key == nil) then
        Barn.errors.keyMissingError(txt);
    end

    if(key ~= nil) then
        btn:SetText( L[key] );
        btn:SetHeight( btn:GetTextHeight() + 2 );

        if( tContains(changeIcon, key) ) then
            btnIcon:SetTexture(icon);
        end
    end
end

Barn.getNPCName = function()
    return GossipFrameNpcNameText:GetText();
end

Barn.errors = {
    localeError = function()
        print("|cFFFF0000[Barn] Locale " .. GetLocale() .. " is not supported.|r");
    end,
    
    gossipShowError = function(err)
        print("|cFFFF0000[Barn] An unexpected exception occurred while opening the barn menu.|r");
        print("|cFFFF0000[Barn] " .. err .. "|r");
    end,
    
    keyMissingError = function(txt)
        print("|cFFFF0000[Barn] Could not find the barn menu key.|r");
        print("|cFFFF0000[Barn] KEY: " .. txt .. "|r");
    end,
    
    initError = function(err)
        print("|cFFFF0000[Barn] An unexpected exception occurred during initialization.|r");
        print("|cFFFF0000[Barn] " .. err .. "|r");
    end,
    
    factionError = function()
        print("|cFFFF0000[Barn] Your faction is neither ALLIANCE or HORDE|r");
    end
};
